<?php

declare(strict_types=1);

use App\Application\Settings\SettingsInterface;
use DI\ContainerBuilder;
use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use Monolog\Processor\UidProcessor;
use Psr\Container\ContainerInterface;
use Psr\Log\LoggerInterface;
use App\Application\TranslationExtension;
use Symfony\Component\Translation\Translator;
use Symfony\Component\Translation\Loader\ArrayLoader;
use Slim\Views\Twig;
use Twig\Loader\FilesystemLoader;

return function (ContainerBuilder $containerBuilder) {
    $containerBuilder->addDefinitions([
        LoggerInterface::class => function (ContainerInterface $c) {
            $settings = $c->get(SettingsInterface::class);

            $loggerSettings = $settings->get('logger');
            $logger = new Logger($loggerSettings['name']);

            $processor = new UidProcessor();
            $logger->pushProcessor($processor);

            $handler = new StreamHandler($loggerSettings['path'], $loggerSettings['level']);
            $logger->pushHandler($handler);

            return $logger;
        },

        Translator::class => function (ContainerInterface $container) {
            $defaultLang = 'en';
            if (session_status() === PHP_SESSION_NONE) {
                session_start(); // Start session if not started
            }
            // Check for language preference in session or cookie
            $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? $defaultLang;

            // Create the Translator instance
            $translator = new Translator($lang);
            $translator->addLoader('array', new ArrayLoader());

            // Load translations from translations.php
            $translations = require __DIR__ . '/translations.php';
            // Set resources for the translator
            foreach ($translations as $locale => $resources) {
                $translator->addResource('array', $resources, $locale);
            }

            return $translator;
        },

        // PDO Database connection
        'db' => function (ContainerInterface $container) {
            $host = 'localhost';
            $dbname = 'jbrshsxb_bulkmail'; // Update with your actual database name
            $username = 'jbrshsxb_mailadmin'; // Update with your database username
            $password = 'Sy5adm!n01'; // Update with your database password

            $pdo = new PDO("mysql:host=$host;dbname=$dbname", $username, $password);
            $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            return $pdo;
        },

        // Twig view setup
        'view' => function (ContainerInterface $container) {
           
            $view = Twig::create(__DIR__ . '/../public/templates', [
                'cache' => false, // Disable cache for development
            ]);

            $translator = $container->get(Translator::class); // Get the Translator instance
            $view->addExtension(new TranslationExtension($translator)); // Pass the Translator instance

            return $view;
        },

    ]);

};
