namespace App\Application\Controllers;

use Slim\Psr7\Request;
use Slim\Psr7\Response;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use PhpOffice\PhpSpreadsheet\IOFactory;
use App\Services\MailService;
use Slim\Views\Twig;

class MailController
{
    private $view;
    private $db;
    
    public function __construct(Twig $view, $db)
    {
        $this->view = $view;
        $this->db = $db;
    }

    public function uploadFile(Request $request, Response $response, array $args)
    {
        if($db)
        {
            console.log("DB is loaded");
        }
        else
        {
            console.log("DB is not loaded");
        }

        $uploadedFiles = $request->getUploadedFiles();
        $file = $uploadedFiles['file'];

        // Ensure file is XLSX
        if ($file->getClientMediaType() !== 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') {
            return $response->withStatus(400)->write('Invalid file type');
        }

        // Save uploaded file to temp directory
        $uploadDir = __DIR__ . '/../../public/uploads/';
        $filename = moveUploadedFile($uploadDir, $file);

        // Process file and save data to DB
        $spreadsheet = IOFactory::load($uploadDir . $filename);
        $data = $spreadsheet->getActiveSheet()->toArray();

        foreach ($data as $row) {
            $mailSender = $this->db->query("SELECT mail_address FROM mail_senders ORDER BY RAND() LIMIT 1")->fetch(PDO::FETCH_ASSOC);

            // Save each row to database
            $stmt = $this->db->prepare("INSERT INTO mail_tasks (mail_reply, mail_email, mail_var1, mail_var2, mail_var3, mail_status)
                                VALUES (?, ?, ?, ?, ?, 'Pending')");

            // Bind the values and execute the statement
            $stmt->execute([
                $mailSender['mail_address'],
                $row[0],
                $row[1],
                $row[2],
                $row[3]
            ]);
        }

        // Queue background processing (using Symfony Process)
        $process = new \Symfony\Component\Process\Process(['php', 'public/index.php', 'mail:process']);
        $process->start();

        // Respond with success and progress bar overlay
        $payload = json_encode(['success' => true, 'message' => 'File uploaded successfully']);
        $response->getBody()->write($payload);

        return $response->withHeader('Content-Type', 'application/json');
    }

    public function getStatus(Request $request, Response $response, array $args)
    {
        // Fetch task progress from DB
        $status = $this->db->query("SELECT COUNT(*) as total, SUM(mail_status = 'Success') as sent, SUM(mail_status = 'Failed') as failed FROM mail_tasks");

        $response->getBody()->write(json_encode($status->fetch()));
        return $response->withHeader('Content-Type', 'application/json');
    }

    function moveUploadedFile($directory, \Slim\Psr7\UploadedFile $uploadedFile)
    {
        $extension = pathinfo($uploadedFile->getClientFilename(), PATHINFO_EXTENSION);
        $basename = bin2hex(random_bytes(8)); // Generate a random filename
        $filename = sprintf('%s.%0.8s', $basename, $extension);
    
        $uploadedFile->moveTo($directory . DIRECTORY_SEPARATOR . $filename);
    
        return $filename;
    }
}
